package com.acme.ejb;

import java.sql.*;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.*;

import com.acme.common.*;
import com.ibm.rmi.javax.rmi.PortableRemoteObject;

/**
 * Bean implementation class for Enterprise Bean: ItemHelper
 */
public class ItemHelperBean implements javax.ejb.SessionBean 
{
	private javax.ejb.SessionContext mySessionCtx;
	private OnlineitemFactory _factory;
	
	/**
	 * Create or register a new item so that it is available to be sold in the 
	 * auction. A number of business rules are run as part of this creation, for
	 * example the item must be assigned a value.
	 * 
	 * @param item the item that is being  created.
	 * @return the status of the result
	 */ 
	public Status create(OnlineitemDataExtended item)
	{
		Status status = new Status();
		try
		{
			long value = 0;
			if (item.getValue() == null)status.addError("The item value can not be null");
			else value = item.getValue().longValue();
			
			if (item.getStartingbid() == null)item.setStartingbid(new Long(value / 2));
			long startingBid = item.getStartingbid().longValue();
			
			if (startingBid > value)
				status.addError("The starting bid must be less than or equal to the value");
				
			if (item.getItemtypeid() == null)
				status.addError("The itemTypeId must be supplied");

			if (item.getCatalognumber() == null)
				item.setCatalognumber(item.getItemtypeid());
				
			if (status.isOk())
			{
				Onlineitem newItem = getFactory().create(item.getItemtypeid());
				newItem.syncOnlineitemData(item);
			}				
		}
		catch (Exception e)
		{
			status.addError(e.toString());
			e.printStackTrace();
		}
		return status;
	}
	private OnlineitemFactory getFactory()
	{
		if (_factory == null)_factory = new OnlineitemFactory();
		return _factory;		
	}
	
	/**
	 * Answer all of the items that are less than or equal to value.
	 * 
	 * @param value the value in cents.
	 * @param currency the type of currency
	 */
	public SearchResults findByValue(long value, int currency)
	{
		SearchResults result = new SearchResults();
		try
		{
			OnlineitemFactory factory = new OnlineitemFactory();
			OnlineitemHome home = factory.acquireOnlineitemHome();				
			Collection col = home.findByValue(value);
			result._results = new SearchResultBean[col.size()];
			Iterator it = col.iterator();
			int i = 0;
			while(it.hasNext())
			{
				Onlineitem item = (Onlineitem)javax.rmi.PortableRemoteObject.narrow(it.next(), Onlineitem.class);
				OnlineitemData data = item.getOnlineitemData();
				SearchResultBean r = new SearchResultBean();
				
				OnlineitemKey key = (OnlineitemKey)data.getPrimaryKey();
				r.setItemTypeId(String.valueOf(key.itemtypeid));
				
				r.setCatalogNumber(String.valueOf(data.getCatalognumber()));
				r.setImageLocnSm(data.getImagelocnsm());
				long nextBid = data.getLastbid() == null
					? data.getStartingbid().longValue()
					: data.getLastbid().longValue() + 1;
				r.setNextBid(Money.asMoneyString(nextBid, currency));
				r.setTitle(data.getTitle());
				r.setValue(Money.asMoneyString(data.getValue(), currency));
				result._results[i++] = r;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			result._status.addError(e.toString());
			result._results = new SearchResultBean[0];
		}
		return result;
	}
	
	/**
	 * Make a bid on the item.
	 * 
	 * @param itemType Id the item that you are bidding on
	 * @param bid the amount of the bid (in US cents)
	 * @param bidder the name of the bidder
	 * @param currency the type of currency to use if the method needs to generate
	 *   messages that include monetary values.
	 */
	public Status makeBid(Integer itemTypeId, Long bid, String bidder, int currency)
	{
		Status status = new Status();
		try
		{
			OnlineitemKey key = new OnlineitemKey(itemTypeId);
			Onlineitem item = getFactory().findByPrimaryKey(key);
			OnlineitemData data = item.getOnlineitemData();
			
			if (data.getLastbid() == null || data.getLastbid().longValue() < bid.longValue())
			{
				if (bid.longValue() < data.getStartingbid().longValue())
				{
					status.addError("The proposed bid of " + Money.asMoneyString(bid, currency) 
						+ " is less than the minimum bid of " 
						+ Money.asMoneyString(data.getStartingbid(), currency));
				}
				else
				{
					data.setLastbid(bid);
					data.setLastbidder(bidder);
					item.syncOnlineitemData(data);
				}
			}
			else
			{
				status.addError("The proposed bid of " + Money.asMoneyString(bid, currency) 
					+ " is not greater than the current bid of " 
					+ Money.asMoneyString(data.getLastbid(), currency));
			}
		}
		catch (Exception e)
		{
			status.addError(e.toString());
			e.printStackTrace();
		}
		return status;
	}
	
	/**
	 * Reserve a number of keys.  These keys are used when creating EJBs.
	 * 
	 * @param numberToReserve the number of keys to reserve
	 * @return the first key that you can use. Return ERROR_KEY if there is some sort of
	 * problem. 
	 */
	public int reserveKeys(int numberToReserve)
	{
		Connection conn = null;
		String sql = null;
		Statement stmt = null;
		int currentKey = ItemHelper.ERROR_KEY;
		try
		{
			conn = ServerHelper.instance().getConnection();
	
			stmt = conn.createStatement();
			sql = "SELECT * FROM EJB.NAMEVALUE WHERE NAME = 'highkey'";
			ResultSet rs = stmt.executeQuery(sql);
			if(rs.next())
			{
				currentKey = rs.getInt("ivalue");
				int highKey = currentKey + numberToReserve;
				sql = "update EJB.NAMEVALUE set ivalue = "
					+ highKey + " WHERE NAME = 'highkey'";
				stmt.execute(sql);
			}
			else
			{
				currentKey = 100;
				int highKey = currentKey + numberToReserve;
				System.err.println("We created an initial highkey of " 
					+ highKey + ", this is only"
					+ " valid if this is a brand new installation");
				sql = "insert into ejb.namevalue values('highkey', "
					+ highKey + ", '')";
				stmt.execute(sql);
			}
	
		}
		catch (SQLException e)
		{
			currentKey = ItemHelper.ERROR_KEY;;
			System.err.println(e.toString());
			System.err.println("sql stmt = " + sql);
		}
		finally
		{
			try	{if(stmt != null)stmt.close();}
			catch (SQLException e2){System.err.println(e2.toString());}
			
			try	{if(conn != null)conn.close();}
			catch (SQLException e2){System.err.println(e2.toString());}
		}
	
		return currentKey;
	}
	
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
}
